﻿using GrapeCity.Documents.Excel;
using System;
using System.Drawing;

namespace DynamicArrayFunctions
{
    class Program
    {
        static Workbook workbook;
        static void Main(string[] args)
        {
            //create a new workbook
            workbook = new Workbook();
            for (int i = 1; i <= 5; i++)
                workbook.Worksheets.Add();

            //Enable Dynamic Formulas
            workbook.AllowDynamicArray = true;

            //Invoke methods to apply dynamic formulas
            Filter();
            RandArray();
            Sequence();
            SortBy();
            Sort();
            Unique();
                
            workbook.Save("DynamicArrayFunctions.xlsx");

            Console.WriteLine("Excel file saved with all functions");
        }

        static void Filter()
        {
            var sheet = workbook.Worksheets[0];
            sheet.Name = "FILTER";
          
            sheet.Range["B3:E12"].Value = new object[,] {
    { "Region", "Sales Rep", "Product", "Units" },
    { "East", "Tom", "Apple", 6380 },
    { "West", "Fred", "Grape", 5619 },
    { "North ", "Amy", "Pear", 4565 },
    { "South", "Sal", "Banana", 5323 },
    { "East", "Fritz", "Apple", 4394 },
    { "West", "Sravan", "Grape", 7195 },
    { "North ", "Xi", "Pear", 5231 },
    { "South", "Hector", "Banana", 2427 },
    { "East", "Tom", "Banana", 4213 },
    { "West", "Fred", "Pear", 3239 }};

            sheet.Range["G3:J4"].Value = new object[,] { { "Criterion", "", "Product", "Units" }, { 5000, null, null, null } };

            sheet.Range["I4"].Formula = "=FILTER(D4:E12,E4:E12>G4,\"\")";

            sheet.Range["E4:E12,G4,I4#"].NumberFormat = "#,##0";
            sheet.Range["B3:E3,G3,I3:J3"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["B3:E3,G3,I3:J3"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["B3:E3,G3,I3:J3"].Font.Color = Color.White;
            sheet.Range["B4:E12,G4,I4#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["E4:E12,G4,I4#"].Borders.LineStyle = BorderLineStyle.Thin;
        }

        static void RandArray()
        {
            var sheet = workbook.Worksheets[1];
            sheet.Name = "RANDARRAY";            
            sheet.Range["D3:H3"].Value = new object[] { "January", "Februray", "March", "April", "May"};
            sheet.Range["$D$4"].Formula = "=RANDARRAY(4,5)";

            sheet.Range["D:H"].EntireColumn.ColumnWidth *= 1.75;            
            sheet.Range["D3:H3"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["D3:H3"].Font.Color = Color.White;
            sheet.Range["D3:H3"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["D4#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["D4#"].Borders.LineStyle = BorderLineStyle.Thin;
        }

        static void Sequence()
        {
            var sheet = workbook.Worksheets[2];
            sheet.Name = "SEQUENCE";

            sheet.Range["D:D"].EntireColumn.ColumnWidth *= 2.5;
            sheet.Range["D3"].Value = "SEQUENCE(10,,100,-10)";
            sheet.Range["$D$4"].Formula = "=SEQUENCE(10,,100,-10)";

            sheet.Range["D3"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["D3"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["D3"].Font.Color = Color.White;            
            sheet.Range["D4#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["D4#"].Borders.LineStyle = BorderLineStyle.Thin;
        }

        static void SortBy()
        {
            var sheet = workbook.Worksheets[3];
            sheet.Name = "SORTBY";

            sheet.Range["$D$4:$D$12"].Value = new object[,] {
    { "Name"},
    { "Fritz"},
    { "Xi"},
    { "Amy"},
    { "Sravan"},
    { "Tom"},
    { "Fred"},
    { "Hector"},
    { "Sal"}
};

            sheet.Range["$E$4:$F$4"].Value = new object[,] { { "Birthday", "" } };
            sheet.Range["$G$4"].Value = "Name";
            sheet.Range["$H$4"].Value = "Birthday";
            sheet.Range["$E$5"].Formula = "=RANDARRAY(COUNTA(SEQUENCE(8)))*40000";
            sheet.Range["$G$5"].Formula = "=SORTBY($D$5:$E$12,$E$5:$E$12)";

            sheet.Range["E:E"].EntireColumn.ColumnWidth *= 1.5;
            sheet.Range["H:H"].EntireColumn.ColumnWidth *= 1.5;
            sheet.Range["$E$5:$E$12,$H$5:$H$12"].NumberFormat = "mm/dd/yyyy";
            sheet.Range["D4:E4, G4:H4"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["D4:E4, G4:H4"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["D4:E4, G4:H4"].Font.Color = Color.White;
            sheet.Range["D5:E12, G5#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["D5:E12, G5#"].Borders.LineStyle = BorderLineStyle.Thin;

            //save to an excel file
            workbook.Save("sortbyfunction.xlsx");
        }

        static void Sort()
        {           
            var sheet = workbook.Worksheets[4];
            sheet.Name = "SORT";

            sheet.Range["$D$4:$G$13"].Value = new object[,] {
    { "Region", "Sales Rep", "Product", "Units"},
    { "East", "Tom", "Apple", 6380d},
    { "West", "Fred", "Grape", 5619d},
    { "North ", "Amy", "Pear", 4565d},
    { "South", "Sal", "Banana", 5323d},
    { "East", "Fritz", "Apple", 4394d},
    { "West", "Sravan", "Grape", 7195d},
    { "North ", "Xi", "Pear", 5231d},
    { "South", "Hector", "Banana", 2427d},
    { "East", "Tom", "Banana", 4213d},
};

            sheet.Range["$I$4:$L$4"].Value = new object[,] {
    { "Region","Sales Rep", "Product", "Units"}
};

            sheet.Range["$I$5"].Formula = "=SORT(D5:G13,4,1,FALSE)";
            sheet.Range["$G$5:$G$13,$L$5:$L$13"].NumberFormat = "#,##0";
            sheet.Range["D4:G4, I4:L4"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["D4:G4, I4:L4"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["D4:G4, I4:L4"].Font.Color = Color.White;
            sheet.Range["D5:G13, I5#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["D5:G13, I5#"].Borders.LineStyle = BorderLineStyle.Thin;
        }

        static void Unique()
        {
            var sheet = workbook.Worksheets[5];
            sheet.Name = "UNIQUE";

            sheet.Range["$B$3:$E$12"].Value = new object[,] {
    { "Region", "Sales Rep", "Product", "Units"},
    { "East", "Tom", "Apple", 6380d},
    { "West", "Fred", "Grape", 5619d},
    { "North ", "Amy", "Pear", 4565d},
    { "South", "Sal", "Banana", 5323d},
    { "East", "Fritz", "Apple", 4394d},
    { "West", "Sravan", "Grape", 7195d},
    { "North ", "Xi", "Pear", 5231d},
    { "South", "Hector", "Banana", 2427d},
    { "East", "Tom", "Banana", 4213d}};

            sheet.Range["$G$3:$I$3"].Value = new object[,] {{"Region", "Sales Rep", "Product"}};

            sheet.Range["$G$4:$I$4"].Formula = "=UNIQUE(B4:B12)";

            sheet.Range["$E$4:$E$12"].NumberFormat = "#,##0";
            sheet.Range["B3:E3, G3:I3"].HorizontalAlignment = HorizontalAlignment.Center;
            sheet.Range["B3:E3, G3:I3"].Interior.Color = Color.FromArgb(68, 114, 196);
            sheet.Range["B3:E3, G3:I3"].Font.Color = Color.White;
            sheet.Range["B4:E12, G4#, H4#, I4#"].Borders.Color = Color.FromArgb(91, 155, 213);
            sheet.Range["B4:E12, G4#, H4#, I4#"].Borders.LineStyle = BorderLineStyle.Thin;
        }
    }
}
